local addon, ns = ...

-- Got Class?
--if not ( select(2,UnitClass("player")) == "SHAMAN" ) then return end
-- Yay!

KokoTotems = LibStub("AceAddon-3.0"):NewAddon("KokoTotems", "AceConsole-3.0")
local libKoko = LibStub('LibKoko-2.0')
local libTotems = LibStub('KokoTotems-1.2')

if not libTotems then return end

local options = { 
	name = "KokoTotems",
	handler = KokoTotems,
	type = "group",
	args = {},
	plugins = {},
}

local view = false

options.args = {
	tInfo = {
		name = "Data Collection",
		handler = tInfo,
		type = "group",
		order = 0,
		args = {
			about = {
				name = "Gathers totem information as you play.\n\n" ..
				"Totem information is gathered and may be used by addon modules. Once the player has cast each totem at least once, this feature can be disabled with no negative impact.\n\n" ..
				"It is recommended to re-enable this feature and Refresh (remove data for totems no longer in game) or Reset (removes all saved data) this information for each major patch.\n\n",
				type = "description",
				order = 1,
			},
			tCol = {
				type = "toggle",
				name = "Gather data",
				order = 2,
				width = 'full',
				desc = "Gathers totem data",
				set = function(info, val) KokoTotems:SetOption("tCol",val) end,
				get = function(info) return KokoTotems:GetOption("tCol") end,
			},
			tClean = {
				type = "execute",
				name = "Refresh",
				order = 4,
				hidden  = function() return true end, --not view end,
				desc = "Refreshes and removes data for totems that are no longer in the game.",
				func = function(info, val) KokoTotems:Cleanup() end,
			},
			tPurge = {
				type = "execute",
				name = "Reset",
				order = 5,
				hidden  = function() return not view end,
				desc = "Deletes all gathered totem information. This turns on data collection.",
				func = function(info, val) KokoTotemCollection_DB = nil; KokoTotems:SetOption("tCol",true) end,
			},
			confirm = {
				type = "toggle",
				name = "Edit Data",
				order = 3,
				width = 'full',
				desc = "Allows Update and Reset of collected data",
				set = function(info, val) view = not view end,
				get = function(info) return view end,
			},
		},
	},
}

function KokoTotems:GetOption(index)
	return KokoTotems_DB[index]
end

function KokoTotems:SetOption(index, val, ...)
	if index == 'tCol' then
		if val then
			libTotems.RegisterCallback(KokoTotems, "TOTEMS_PTU_NEW", function(event, slot, name, startTime, duration, icon, spellid) 
				if not KokoTotemCollection_DB then
					KokoTotemCollection_DB = {}
				end
				--print(event, slot, name, startTime, duration, icon, spellid)
				if duration and spellid then
					if not KokoTotemCollection_DB[spellid] then
						-- new data, announce information
						libTotems.callbacks:Fire('TOTEMS_GATHERED_NEW', spellid, name, icon, duration)
					end
					KokoTotemCollection_DB[spellid] = { name, icon, duration}
				end
			end)
			else
			libTotems.UnregisterCallback(KokoTotems, "TOTEMS_PTU_NEW")
		end
	end
	
	KokoTotems_DB[index] = val
end

function KokoTotems:TotemCollection()  
	return libKoko:CopyTable(KokoTotemCollection_DB)
end

function KokoTotems:OnInitialize()
	-- Called when the addon is loaded
	if not KokoTotems_DB then KokoTotems_DB = { ['tCol'] = true } end
	
	libKoko:RegisterAddon("KokoTotems",options)
	LibStub("AceConfig-3.0"):RegisterOptionsTable("KokoTotems", options)
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("KokoTotems", "Koko Totems")
	
	self:SetOption('tCol', self:GetOption('tCol'))
	
	-- Chat Commands
	self:RegisterChatCommand("ktt", "ChatCommand")
	
	-- cleanup old totem data no longer in game
	self:Cleanup()
	
	-- LDB launcher
	LibStub:GetLibrary("LibDataBroker-1.1"):NewDataObject("KokoTotems", {
		type = "launcher",
		icon = "Interface\\Icons\\Spell_Nature_TremorTotem.blp",
		text = "Koko Totem Timers",
		OnClick = function(clickedframe, button)
			LibStub("AceConfigDialog-3.0"):Open("KokoTotems")
		end,
		OnLeave = function() return GameTooltip:Hide() end,
		OnEnter = function(self) 
			local frame = self
			local showBelow = select( 2, self:GetCenter() ) > UIParent:GetHeight() / 2
			GameTooltip:SetOwner( self, "ANCHOR_NONE" )
			GameTooltip:SetPoint( showBelow and "TOP" or "BOTTOM", self, showBelow and "BOTTOM" or "TOP" )
			GameTooltip:AddLine( "Click to show options" )
			return GameTooltip:Show()
		end,
	})
end

function KokoTotems:ChatCommand(input)
	if not input or input:trim() == "" then
		LibStub("AceConfigDialog-3.0"):Open("KokoTotems")
		else
        LibStub("AceConfigCmd-3.0").HandleCommand(KokoTotems, "KokoTotems", input)
	end
end

function KokoTotems:Cleanup()
	if not KokoTotemCollection_DB then return end
	
	for id, _ in pairs(KokoTotemCollection_DB) do
		if not GetSpellInfo(id) then
			KokoTotemCollection_DB[id] = nil
		end
	end
end