local version = "KokoTotems-1.2"

assert(LibStub, version .." requires LibStub")
assert(LibStub:GetLibrary("CallbackHandler-1.0", true), version .. " requires CallbackHandler-1.0")

local lib, oldminor = LibStub:NewLibrary(version, 1)
if not lib then return end
oldminor = oldminor or 0

lib.callbacks = lib.callbacks or LibStub:GetLibrary("CallbackHandler-1.0"):New(lib)

local K_MAX_TOTEMS = 5
local totems = {}
local startupFunc = {}

local frame = CreateFrame("Frame")
frame:SetScript('OnEvent', function(self, event, ...) self[event](self, ...) end)
frame:RegisterEvent('PLAYER_LOGIN')
frame:RegisterEvent('PLAYER_ENTERING_WORLD')
frame:RegisterEvent('PLAYER_TOTEM_UPDATE')

function frame:PLAYER_LOGIN()
   for _,func in ipairs(startupFunc) do
      func()
   end
end

function frame:PLAYER_ENTERING_WORLD()
   lib:ForceUpdate()
end

function frame:PLAYER_TOTEM_UPDATE(slot, ...)  
	local _, name, startTime, duration, icon = GetTotemInfo(slot)
	local _,_,_,_,_,_, spellid = GetSpellInfo(name)
   
	if (duration > 0 ) then
		-- totem dropped
		local est_dur = startTime + duration - GetTime()
		
		-- blizz icon file number to string
		if type(icon) == "number" then
			icon = tostring(icon)
		end
		
		totems[slot] = {name, duration, icon, spellid, startTime}
		lib.callbacks:Fire("TOTEMS_PTU_NEW", slot, name, est_dur, duration, icon, spellid)
	else
		-- totem destroyed
		name, _, icon, spellid = lib:GetSavedTotem(slot)
      
		totems[slot] = nil;
		lib.callbacks:Fire("TOTEMS_PTU_DESTROY", slot, name, icon, spellid)
	end
end

function lib:GetSavedTotem(slot)
   if totems[slot] then
      return unpack(totems[slot])
   else
      return nil
   end
end

function lib:GetMaxTotems()
   return K_MAX_TOTEMS
end

function lib:RegisterStartup(func)
   assert(type(func) == 'function', 'Must pass a function')
   tinsert(startupFunc, func)
end

function lib:ForceUpdate()
   for slot=1,K_MAX_TOTEMS,1 do
      frame:PLAYER_TOTEM_UPDATE(slot)
   end
end