local version = 'LibKoko-2.0'

assert(LibStub, version .. " requires LibStub")

local lib, oldminor = LibStub:NewLibrary(version, 1)
if not lib then return end
oldminor = oldminor or 0

local addon_list = {}

function lib:CopyTable(tbl)
	if not tbl then return {} end
	
	local copy = {}
	for k, v in pairs(tbl) do
		if (type(v) == "table") then
			copy[k] = CopyTable(v)
			else
			copy[k] = v
		end
	end
	return copy
end

function lib:RegisterAddon(addon, optMenu)
	-- avoid duplicate addon keys
	if not (addon and optMenu) then return false end
	if addon_list[addon] then
		print('Attempting to register addon with duplicate key', addon)
		return false
	end
	
	addon_list[addon] = optMenu
	return true
end

function lib:RegisterAddonPlugIn(addon, plugin, optMenu)
	-- avoid duplicate addon keys
	if not (addon and plugin and optMenu) then return false end
	if addon_list[addon] then
		if not addon_list[addon].plugins then
			addon_list[addon].plugins = {}
		end
		if addon_list[addon].plugins[plugin] then 
			print('Attempting to register addon plugin with duplicate key', addon, plugin)
			return false
		else
			addon_list[addon].plugins[plugin] = optMenu
			return true
		end
	end
end

function lib:UpdateAddonPlugIn(addon, plugin, optMenu)
	-- avoid duplicate addon keys
	if not (addon and plugin and optMenu) then return false end
	if addon_list[addon] then
		if not addon_list[addon].plugins then
			addon_list[addon].plugins = {}
		end
		if addon_list[addon].plugins[plugin] then 
			addon_list[addon].plugins[plugin] = optMenu
			return true
		else
			print('Addon plugin is not registered', addon, plugin)
			return false
		end
	end
end

function lib:AddonListing()
	local set = {}
	for i,_ in pairs(addon_list) do
		tinsert(set,i)
	end
	return set
end

function lib:AddonPluginsListing(addon)
	local set = {}
	for i,_ in pairs(addon_list[addon].plugins) do
		tinsert(set,i)
	end
	return set
end