local addon, ns = ...

-- Got Class?
if not ( select(2,UnitClass("player")) == "SHAMAN" ) then return end
-- Yay!

local KokoTotems = KokoTotems
local media = LibStub("LibSharedMedia-3.0")

if not KokoTotems then return end

local function SoundListing()
	local hash = media:HashTable('sound')
	local list = {}
	
	for index, val in pairs(hash) do
		list[index] = index
	end
	
	return list
end

ns.ReBuildMenu = function()
	ns.menu.stomp.args.totemCfg.args = {}
	ns.BuildMenu()
end

ns.BuildMenu = function()
	-- build individual totem options
	local collection = KokoTotems:TotemCollection()
	
	for i, v in pairs(collection) do
		ns.AddTotemToMenu(i,v[1])
	end
end

ns.AddTotemToMenu = function(totemID, totemName)
	ns.menu.stomp.args.totemCfg.args[totemName] = ns.BuildTotemOption(totemID, totemName)
end

ns.BuildTotemOption = function(totemID, totemName)
	local menuOpt = {
		name = totemName,
		type = 'group',
		order = totemID,
		hidden = function() return not IsPlayerSpell(totemID) end,
		args = {
			mode = {
				order = 1,
				type = "select",
				values = { "None", "Always", "Combat", "PvP" },
				name = "Filter Mode",
				desc = "Filters when warning occurs",
				set = function(info,val)  ns.SetTotemOption(totemID,'mode',val) end,
				get = function(info) return ns.GetTotemOption(totemID,'mode') end  
			},
			sound = {
				order = 2,
				type = "select",
				values = SoundListing,
				name = "Sound",
				desc = "Play a sound on totem death",
				set = function(info,val)  ns.SetTotemOption(totemID,'sound',val); PlaySoundFile(media:Fetch("sound", val)) end,
				get = function(info) return ns.GetTotemOption(totemID,'sound') end  
			},
			chat = {
				order = 3,
				type = "toggle",
				name = "Party chat",
				desc = "Announce deaths to party chat",
				set = function(info,val)  ns.SetTotemOption(totemID,'chat',val) end,
				get = function(info) return ns.GetTotemOption(totemID,'chat') end  
			},
			flash = {
				order = 4,
				type = "toggle",
				name = "Screen flash",
				desc = "Enables a screen flash on totem death",
				set = function(info,val)  ns.SetTotemOption(totemID,'flash',val) end,
				get = function(info) return ns.GetTotemOption(totemID,'flash') end  
			},
			raid = {
				order = 5,
				type = "toggle",
				name = "Raid Warning",
				desc = "Display a Raid Warning on totem death. Note that only you will see this message",
				set = function(info,val)  ns.SetTotemOption(totemID,'raid',val) end,
				get = function(info) return ns.GetTotemOption(totemID,'raid') end  
			},
			--[[
				icon = {
				order = 6,
				type = "toggle",
				name = "Icon Warning",
				desc = "Displays a pop up icon on totem death",
				set = function(info,val)  ns.SetTotemOption(totemID,'icon',val) end,
				get = function(info) return ns.GetTotemOption(totemID,'icon') end  
				},
			--]]
		},
	}
	return menuOpt
end

-- options menu
do
	local menu = {
		name = 'Stomp Alerts',
		type = "group",
		order = 3,
		childGroups = 'tab',
		args = {
			mydesc = {
				order = 1,
				type = "description",
				name =   "Minimalistic totem death alerts.\n\n",
			},
			general = {
				order = 2,
				type = 'group',
				name = 'General settings',
				args = {
					enable = {
						order = 1,
						type = "toggle",
						name = "Enable",
						width = 'full',
						desc = "Enable/disables death alerts",
						set = function(info,val) ns.SetGlobalInfo('enable',val) end,
						get = function(info) return ns.GetGlobalInfo('enable') end
					},
					refresh = {
						type = 'execute',
						name = 'Refresh',
						order = 2,
						hidden = true,
						desc = 'Refreshes and purges settings for totems not collected.',
						func = function()
							ns.Purge()
						end,
					},
					reset = {
						type = 'execute',
						name = 'Reset',
						order = 3,
						desc = 'Reset settings to default values',
						func = function()
							ns.Reset()
						end,
					},
				},
			},
			totemCfg = {
				order = 3,
				type = 'group',
				name = 'Totem settings',
				args = {},
			},
		},
	} 
	ns.menu = {
		stomp = menu,
	}
end

----------------------------------------------
----		Configuration functions 			----
----------------------------------------------

ns.GetGlobalInfo = function(index)
	return KokoTotemsStomp_DB[index]
end

ns.SetGlobalInfo = function(index,val)
	KokoTotemsStomp_DB[index] = val
	if index == 'enable' then
		ns.Enable(val)
	end
	
end

ns.SetTotemOption = function(totem, index, val)
	--if not KokoTotemsStomp_DB.totems[totem] then return end
	
	KokoTotemsStomp_DB.totems[totem][index] = val
end

ns.GetTotemOption = function(totem, index)
	--if not KokoTotemsStomp_DB.totems[totem] then return end
	
	return KokoTotemsStomp_DB.totems[totem][index]
end