local addon, ns = ...

-- Got Class?
if not ( select(2,UnitClass("player")) == "SHAMAN" ) then return end
-- Yay!

local KokoTotems = KokoTotems --LibStub:GetAddon("KokoTotems")
local libKoko = LibStub:GetLibrary('LibKoko-2.0')
local libTotems = LibStub:GetLibrary('KokoTotems-1.2')
local LibEffects = LibStub:GetLibrary('LibEffects-1.0')
local media = LibStub("LibSharedMedia-3.0")

if not KokoTotems then return end
if not libTotems then return end

-- menu updates
local frame = CreateFrame('Frame')
frame:SetScript('OnEvent', function(self, event, ...) self[event](self, ...) end)
frame:RegisterEvent('PLAYER_SPECIALIZATION_CHANGED')
frame:RegisterEvent('PLAYER_TALENT_UPDATE')

function frame:PLAYER_SPECIALIZATION_CHANGED()
	LibStub("AceConfigRegistry-3.0"):NotifyChange("KokoTotems")
end

function frame:PLAYER_TALENT_UPDATE()
	LibStub("AceConfigRegistry-3.0"):NotifyChange("KokoTotems")
end

ns.DefaultTotemSettings = function()
	local settings = {
		['mode'] = 1,
		['sound'] = 'None',
		['chat'] = false,
		['flash'] = false,
		['raid'] = false,
		['icon'] = false,         
	}
	return settings   
end

ns.InitDB = function(val)
	if not KokoTotemsStomp_DB or val then
		KokoTotemsStomp_DB = {
			['enable'] = true,
			['icon'] = {
				['x'] = 500,
				['y'] = 500,
			},
			['totems'] = {}
		}
		
		-- import totems
		local collection = KokoTotems:TotemCollection()
		
		for i, v in pairs(collection) do
			KokoTotemsStomp_DB.totems[i] = ns.DefaultTotemSettings()
		end
	end  
end

ns.Enable = function(val)
	if val then
		-- register callback events
		libTotems.RegisterCallback(ns, "TOTEMS_PTU_DESTROY")
		else
		-- unregister callback events
		libTotems.UnregisterCallback(ns, "TOTEMS_PTU_DESTROY")
	end
end

-------------

ns.InitAll = function()
	--KokoTotemsStomp_DB = nil
	ns.InitDB()
	ns.Purge()
	ns.BuildMenu()
	ns.Enable(KokoTotemsStomp_DB.enable)
	
	libTotems.RegisterCallback(ns, 'TOTEMS_GATHERED_NEW')
end

ns.Reset = function()
	ns.InitDB(true)
end

ns.Purge = function()
	local totemMenu = ns.menu.stomp.args.totemCfg.args
	local collection = KokoTotems:TotemCollection()
	
	for i, _ in pairs(KokoTotemsStomp_DB.totems) do
		
		if not collection[i] then
			totemMenu[i] = nil
			KokoTotemsStomp_DB.totems[i] = nil	
		end
	end
end


function ns:TOTEMS_GATHERED_NEW(event, spellid, name, ...)
	KokoTotemsStomp_DB.totems[spellid] = ns.DefaultTotemSettings() 
	ns.AddTotemToMenu(spellid, name)
	LibStub("AceConfigRegistry-3.0"):NotifyChange("KokoTotems")
end

function ns:TOTEMS_PTU_DESTROY(event, slot, name, icon, spellid)
	if ns.CheckFilter(spellid) then
		local totem = KokoTotemsStomp_DB.totems[spellid]
		local tex = name
		if icon then
			tex = " |T"..icon..":20:20:-5|t" .. tex
		end
		-- sound
		if totem.sound ~= 'None' then
			PlaySoundFile(media:Fetch('sound', totem.sound))
		end
		if totem.chat then
			if ( GetNumGroupMembers(LE_PARTY_CATEGORY_HOME) > 0 ) then
				SendChatMessage(name .. " Destroyed." ,"PARTY")
			elseif ( GetNumGroupMembers(LE_PARTY_CATEGORY_INSTANCE) > 0 ) and ( not IsInRaid() ) then
				SendChatMessage(name .. " Destroyed." ,"INSTANCE_CHAT")
			else
				print(tex .. " Destroyed")
			end
		end
		if totem.flash then
			LibEffects:Flash()
		end
		if totem.raid then
			local color = {r = 1, g = 1, b = 1} 
			RaidNotice_AddMessage(RaidWarningFrame, tex .. ' Destroyed', color);
		end
		if totem.icon then
			local color = {r = 1, g = 1, b = 1} 
			--RaidNotice_AddMessage(KokoIconF, tex .. ' Destroyed', color);
		end
	end
end

-- libTotems
libTotems:RegisterStartup(ns.InitAll)
libKoko:RegisterAddonPlugIn("KokoTotems", 'Stomp Alerts', ns.menu)

--
ns.CheckFilter = function(totem)
	if KokoTotemsStomp_DB.totems[totem] then
		local mode = KokoTotemsStomp_DB.totems[totem].mode
		
		if mode == 1 then
			return false
		elseif mode == 2 then
			return true
		elseif mode == 3 then
			if UnitAffectingCombat("player") then return true end
		elseif mode == 4 then
			if UnitIsPVP("player") then return true end
		end
	end
return false
end

