local addon, ns = ...

-- Got Class?
--if not ( select(2,UnitClass("player")) == "SHAMAN" ) then return end
-- Yay!

local media = LibStub("LibSharedMedia-3.0")

local function TextureListing()
	local hash = media:HashTable('statusbar')
	local list = {}
	
	for index, val in pairs(hash) do
		list[index] = index
	end
	
	return list
end

local function FontListing()
	local hash = media:HashTable('font')
	local list = {}
	
	for index, val in pairs(hash) do
		list[index] = index
	end
	
	return list
end

local IconLocation = {
	['LEFT'] = 'LEFT',
	['RIGHT'] = 'RIGHT',
}

local BarGrowth = {
	['UP'] = 'UP',
	['DOWN'] = 'DOWN',
}

-- options menu
do
	local menu = {
		name = 'Timer Bar',
		type = "group",
		childGroups = 'tab',
		order = 2,
		args = {
			mydesc = {
				order = 1,
				type = "description",
				name =   "Simple shaman totem timer bars inspired by GotWood.\n\n",
			},
		},
	}
	
	local options = {}
	
	options['globalopts'] = {
		order = 1,
		type = "group",
		name = "Global",
		args = {
			enable = {
				order = 1,
				type = "toggle",
				name = "Enable",
				desc = "Enable/disables totem timers",
				set = function(info,val) ns.SetGlobalInfo('enabled',val) end,
				get = function(info) return ns.GetGlobalInfo('enabled') end
			},
			show_anchor = {
				order = 2,
				type = "toggle",
				name = "Anchor",
				desc = "Show/Hides anchors",
				set = function(info,val) ns.SetGlobalInfo('anchor',val) end,
				get = function(info) return ns.GetGlobalInfo('anchor') end
			},
			reset = {
				type = "execute",
				order = -1,
				name = "Reset",
				desc = "Resets settings back to the default values",
				func = function(info,val)
					KokoTotemsTimers_DB = nil
					ns.InitAll()
				end,
			},
		},
	}
	
	options['display'] = {
		type = "group",
		name = "Display",
		order = 2,
		args = {
			texture = {
				type = "select",
				name = "Texture",
				order = 1,
				values = function(info) return TextureListing() end,
				set = function(info,val) ns.SetGlobalInfo('texture',val) end,
				get = function(info) return ns.GetGlobalInfo('texture') end
			},
			font = {
				type = "select",
				name = "Font",
				order = 2,
				values = function(info) return FontListing() end,
				set = function(info,val) ns.SetGlobalInfo('font',val) end,
				get = function(info) return ns.GetGlobalInfo('font') end
			},
			font_size = {
				type = "range",
				name = "Font Size",
				order = 3,
				min = 8,
				max = 25,
				step = 1,
				set = function(info,val) ns.SetGlobalInfo('font_size',val) end,
				get = function(info) return ns.GetGlobalInfo('font_size') end
			},
		},
	}
	
	options['config'] = {
		type = "group",
		name = "Bars",
		order = 3,
		args = {},
	}
	options['config'].args['timer'] = {
		type = "group",
		name = "Timer Bars",
		order = 1,
		desc = "Totem timer configure options",
		args = {
			icon = {
				order = 2,
				name = 'Icon Location',
				type = 'select',
				values = IconLocation,
				set = function(info,val) ns.SetBarInfo('timer','icon',val) end,
				get = function(info) return ns.GetBarInfo('timer','icon') end
			},
			growth = {
				order = 3,
				name = 'Bar Growth',
				type = 'select',
				values = BarGrowth,
				set = function(info,val) ns.SetBarInfo('timer','growth',val) end,
				get = function(info) return ns.GetBarInfo('timer','growth') end
			},
			scale = {
				type = "range",
				name = "Scale",
				order = 8,
				desc = "Timer bar scale",
				min = 0.5,
				max = 2,
				step = 0.1,
				set = function(info,val) ns.SetBarInfo('timer','scale',val) end,
				get = function(info) return ns.GetBarInfo('timer','scale') end
			},
			width = {
				type = "range",
				name = "Width",
				order = 9,
				desc = "Timer bar width",
				min = 50,
				max = 500,
				step = 10,
				set = function(info,val) ns.SetBarInfo('timer','width',val) end,
				get = function(info) return ns.GetBarInfo('timer','width') end
			},
		},
	}
	for i, v in pairs(options) do
		menu.args[i] = v
	end
	
	ns.menu = {
		timers = menu,
	}
end
----------------------------------------------
----		Configuration functions 			----
----------------------------------------------


ns.GetGlobalInfo = function(index)
	if not index then return end
	
	return KokoTotemsTimers_DB[index]
end

ns.SetGlobalInfo = function(index, val)
	KokoTotemsTimers_DB[index] = val
	
	-- index specific stuff
	if ns.Settings[index] then
		ns.Settings[index](val)
	end
end

ns.GetBarInfo = function(bar, index)
	if not (index and bar) then return end
	
	return KokoTotemsTimers_DB[bar][index]
end

ns.SetBarInfo = function(bar,index, val)
	KokoTotemsTimers_DB[bar][index] = val
	
	-- index specific stuff
	local GTB = {
		['timer'] = ns.GTBTotem
	}
	
	if ns.BarSettings[index] then
		ns.BarSettings[index](GTB[bar],val)
	end
end

ns.Settings = {
	['enabled'] = function()
		ns.InitAll()
		LibStub:GetLibrary('KokoTotems-1.0'):ForceUpdate()
		--lib
	end,
	['texture'] = function(val)
		ns.SetGlobalInfo('texpath', media:Fetch('statusbar', val, true))
		ns.SetTexture()
	end,
	['font'] = function(val)
		ns.SetGlobalInfo('fontpath', media:Fetch('font', val, true))
		ns.SetFont()
	end,
	['font_size'] = function(val)
		ns.SetFont()
	end,
	['anchor'] = function(val)
		ns.SetAnchor(val)
	end,
	['index'] = function(val)
		
	end,
}

ns.BarSettings = {
	['icon'] = function(bar,val)
		bar:SetIconPosition(val)
	end,
	['growth'] = function(bar,val)
		bar:SetBarGrowth(val)
	end,
	['scale'] = function(bar,val)
		bar:SetScale(val)
	end,
	['width'] = function(bar,val)
		bar:SetWidth(val)
	end,
}
